@@ -1,10 +1,8 @@
 /*
- * $Xorg: parsedpy.c,v 1.4 2001/02/09 02:05:38 xorgcvs Exp $
- * $XdotOrg: $
  *
  * parse_displayname - utility routine for splitting up display name strings
  *
- * 
+ *
 Copyright 1989, 1998  The Open Group
 
 Permission to use, copy, modify, distribute, and sell this software and its
@@ -30,8 +28,6 @@ in this Software without prior written authorization from The Open Group.
  * Author:  Jim Fulton, MIT X Consortium
  */
 
-/* $XFree86: xc/programs/xauth/parsedpy.c,v 3.7 2003/07/09 15:27:37 tsi Exp $ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -39,13 +35,13 @@ in this Software without prior written authorization from The Open Group.
 #include <stdio.h>			/* for NULL */
 #include <ctype.h>			/* for isascii() and isdigit() */
 #include <X11/Xos.h>			/* for strchr() and string routines */
-#include <X11/Xlib.h>			/* for Family contants */
-#ifdef hpux
-#include <sys/utsname.h>		/* for struct utsname */
-#endif
+#include <X11/Xlib.h>			/* for Family constants */
 #include <X11/Xauth.h>			/* for FamilyLocal */
 #include <X11/Xmu/SysUtil.h>
 
+#include <sys/stat.h>
+#include <limits.h>
+
 #if defined(UNIXCONN) || defined(LOCALCONN)
 #define UNIX_CONNECTION "unix"
 #define UNIX_CONNECTION_LENGTH 4
@@ -59,14 +55,14 @@ in this Software without prior written authorization from The Open Group.
  */
 
 char *
-copystring (char *src, int len)
+copystring (const char *src, int len)
 {
     char *cp;
 
     if (!src && len != 0) return NULL;
     cp = malloc (len + 1);
     if (cp) {
-	if (src) strncpy (cp, src, len);
+	if (src) memcpy (cp, src, len);
 	cp[len] = '\0';
     }
     return cp;
@@ -87,7 +83,7 @@ copyhostname (void)
 {
     char buf[256];
 
-    return (get_local_hostname (buf, sizeof buf) ? 
+    return (get_local_hostname (buf, sizeof buf) ?
 	    copystring (buf, strlen (buf)) : NULL);
 }
 #endif
@@ -95,8 +91,8 @@ copyhostname (void)
 /*
  * parse_displayname - display a display string up into its component parts
  */
-Bool 
-parse_displayname (char *displayname, 
+Bool
+parse_displayname (const char *displayname,
 		   int *familyp,	/* return */
 		   char **hostp,	/* return */
 		   int *dpynump,	/* return */
@@ -162,8 +158,32 @@ parse_displayname (char *displayname,
 
     if (!host) return False;
 
-    if(strncmp (host, "/tmp/launch", 11) == 0) {
-        family = FamilyLocal;
+    {
+        /*
+         * If using launchd socket, remove the screen number from the end
+         * of $DISPLAY and check if it is a path to a socket.
+         */
+        char path[PATH_MAX];
+        struct stat sbuf;
+
+#ifdef HAVE_STRLCPY
+        strlcpy(path, displayname, sizeof(path));
+#else
+        strncpy(path, displayname, sizeof(path) - 1);
+        path[sizeof(path) - 1] = '\0';
+#endif
+        if (0 == stat(path, &sbuf) && S_ISSOCK(sbuf.st_mode)) {
+            family = FamilyLocal;
+        } else {
+            char *dot = strrchr(path, '.');
+            if (dot) {
+                *dot = '\0';
+                /* screen = atoi(dot + 1); */
+                if (0 == stat(path, &sbuf) && S_ISSOCK(sbuf.st_mode)) {
+                    family = FamilyLocal;
+                }
+            }
+        }
     }
 
     /*
@@ -184,7 +204,7 @@ parse_displayname (char *displayname,
 	    free (host);
 	    return False;
 	}
-	
+
 	dpynum = atoi (ptr);		/* it will handle num. as well */
 	ptr = cp;
     }
@@ -236,4 +256,4 @@ parse_displayname (char *displayname,
     return True;
 }
 
-	    
+
